(in-package "GWB")

(defvar *postscript-viewer-program-name* "gv")

(defun print-strategy-history (displayer strat stream)
  "Print the history of a strategy using a display function."
  (declare (type strategy strat))
  (funcall displayer
           nil 
           (strategy-history strat)
           (history-components (strategy-history strat))
           stream))

(defun simple-printer (player h c s)
  "Simple text-based history printer."
  (declare (type hist h) (ignore c))
  (cond
    ((null h) nil)
    (t
     (simple-printer (not player) (cdr h) nil s)
     (let ((move (car h)))
       (format s "~&~A ~A ~A ~A ~A~%"
               (if player 'p 'o)
               (if (questionp move) 'q 'a)
               (if (equal '#* (move-comp move))
                   "-"
                 (comp-name (move-comp move)))
               (token-name (move-token move))
               (and (move-just move) (move-position (move-just move) h)))))))

(defun history-components (h &optional (names '()))
  "Find the list of components used in a history.
The result is a sorted list of entries of the form (component column . name)"
  (declare (type hist h) (list names))
  (let ((components '()))
    (dolist (m h)
      (declare (type move m))
      (unless (member (move-comp m) components :test #'equal)
        (push (move-comp m) components)))
    (setf components (sort components #'comp-<))
    (let ((n 0))
      (mapcar
       #'(lambda (c)
           (declare (type comp c))
           (list*
            c
            (prog1 n (incf n))
            (or (cdr (assoc c names)) (comp-name c))))
       components))))

(defun find-component (move components)
  (declare (type move move))
  (if (null components)
      nil
    (if (wherep* move (car (car components)))
        (car components)
      (find-component move (cdr components)))))

(defun tabular-printer (player h components s)
  "Tabular text-based history printer."
  (declare (type hist h))
    (labels ((printer (player h)
             (cond
               ((null h) nil)
               (t
                (printer (not player) (cdr h))
                (let* ((move (car h))
                       (comp (find-component move components)))
                  (declare (type move move))
                  (format s "~&~A~A~%"
                          (make-string (* 2 (cadr comp))
                                       :initial-element #\space)
                          (move-name move (car comp))))))))
      (printer player h)))

(defun readable-time ()
  (multiple-value-bind (second minute hour date month year)
      (decode-universal-time (get-universal-time))
    (format nil "~A/~A/~A ~A:~A:~A" date month year hour minute second)))

(defun eps-printer (player h components s)
  "Encapsulated Postscript history printer"
  (declare (type hist h))
  (let ((lines (length h))
        (colsize 36)
        (linesize 18)
        (fontsize 10))
    (format s
            "~&%!PS-Adobe-3.0 EPSF-3.0~%~
               %%Creator: The Games Workbench~%~
               %%DocumentFonts: Times-Roman~%~
               %%CreationDate: ~A~%~
               %%BoundingBox: ~D ~D ~D ~D~%"
            (readable-time)
            0 0
            (ceiling (* (+ 2 (length components)) colsize))
            (ceiling (* (+ 2 lines) linesize)))
    (format s "~&%%EndComments~%")
    (format s
            "~&/FF1 /Times-Roman findfont ~F scalefont def~%" fontsize)
    (format s "~&/F1 {FF1 setfont} bind def~%")
    (format s
            "~&/FF2 /Times-Bold findfont ~F scalefont def~%" fontsize)
    (format s "~&/F2 {FF2 setfont} bind def~%")
    (format s
            "~&/trans {~F mul ~F add exch ~F mul ~F add exch} bind def~%"
            linesize linesize
            colsize (* 1.5 colsize))
    (format s
            "~&/trans2 {~F mul ~F add exch ~F mul ~F add exch} bind def~%"
            linesize linesize
            colsize (* 1.6 colsize))
    (format s
            "~&/rshow {dup stringwidth exch neg exch neg rmoveto show}~
             bind def~%")
    (format s "~&.6 setlinewidth~%")
    (format s "~&F2~%")
    (dolist (entry components)
      (format s "~&~F ~F trans moveto (~A) rshow~%"
              (cadr entry) lines
              (cddr entry)))
    (labels
        ((printer (player h n)
           (cond
             ((null h) nil)
             (t
              (printer (not player) (cdr h) (+ n 1))
              (let* ((move (car h))
                     (comp (find-component move components))
                     (just-position
                      (if (move-just move)
                          (move-position (move-just move) h)))
                     (just-comp
                      (if (move-just move)
                          (find-component (move-just move)
                                          components))))
                (format s
                        "~&~F ~F trans moveto F1 (~A) rshow~%"
                        (cadr comp)
                        n
                        (move-name move (car comp)))
                (when just-position
                  (let ((x (cadr comp))
                        (y n)
                        (xj (cadr just-comp))
                        (yj (+ n just-position)))
                    (format s
                            "~&~F ~F trans2 moveto ~
                             ~F ~F trans2 ~F ~F trans2 ~F ~F trans2 ~
                             curveto stroke~%"
                            x y
                            (+ (max x xj) 1/2)
                            (+ y (/ (- yj y) 4))
                            (+ (max x xj) 1/2)
                            (+ y (* 3/4 (- yj y)))
                            xj yj))))))))
      (printer player h 0)))
  (format s "~&showpage~%")
  (format s "~&%%EOF~%"))
